import argparse

from rich.console import Console
from rich.table import Table

from . import get_sensors, SensorTail


def print_sensors():
    sensors = get_sensors()

    console = Console()

    table = Table(show_header=True, header_style="bold magenta")
    table.add_column("ID")
    table.add_column("Raspberry Shake Ref.")
    table.add_column("Type")
    table.add_column("Latitude")
    table.add_column("Longitude")
    table.add_column("Online")
    
    for sensor in sensors:
        table.add_row(
            str(sensor.id),
            sensor.secondary_id,
            sensor.type,
            str(sensor.publicLocation and sensor.publicLocation[0]),
            str(sensor.publicLocation and sensor.publicLocation[1]),
            str(sensor.online),
        )

    console.print(table)

def print_sensors_csv():
    sensors = get_sensors()

    for sensor in sensors:
        print(
            sensor.id,
            sensor.secondary_id,
            sensor.type,
            sensor.publicLocation and sensor.publicLocation[0],
            sensor.publicLocation and sensor.publicLocation[1],
            sensor.online,
            sep=","
        )

def tail(sensor_id: int):
    sensor_tail = SensorTail(sensor_id)

    for datagram in sensor_tail:
        print(datagram.channel, datagram.timestamp, *datagram, sep=",")


def main():
    parser = argparse.ArgumentParser(description='CLI for fetching data from the CRISiSLab seismometer network')
    parser.add_argument('--list', action='store_true', help='List all nodes in the network')
    parser.add_argument('--tail', type=int, help='Follow live data from node')
    parser.add_argument('--csv', action='store_true', help='Output data in CSV format')
    args = parser.parse_args()

    if not args.list and not args.tail:
        parser.print_help()

    if args.list:
        if args.csv:
            print_sensors_csv()
        else:
            print_sensors()
    elif args.tail:
        tail(args.tail)


import requests
from typeguard import typechecked
from .models import Sensor
from typing import Any, List

API_URL = "https://shakenet-manager.viggers.workers.dev/api/v0/sensors"

@typechecked
def get_sensors(api_url: str = API_URL) -> List[Sensor]:
    """Get a list of sensors from the ShakeNet API.

    Parameters
    ----------
    api_url : str
        The URL of the ShakeNet API endpoint.

    Returns
    -------
    list
        A list of sensors.
    """
    response = requests.get(api_url)
    response.raise_for_status()

    json: dict[str, dict[Any, Any]] = response.json()["sensors"]

    sensors = [Sensor(**sensor) for sensor in json.values()]

    return sensors


from typing import Iterator, Optional
from typeguard import typechecked
from websocket import WebSocket 
import msgpack

@typechecked
class Datagram:
    """Class to represent a single datagram from a sensor.

    Parameters
    ----------
    channel : str
        The seismic channel the datagram is from.
    timestamp : float
        The time of the first sample in the datagram in seconds since the epoch.
    data : list[int]
        The samples in the datagram.
    """

    channel: str
    timestamp: float
    data: list[int]

    def __init__(self, channel: str, timestamp: float, data: list[int]):
        self.channel = channel
        self.timestamp = timestamp
        self.data = data

    def __iter__(self) -> Iterator[int]:
        return iter(self.data)
    
    def __str__(self) -> str:
        return f"{self.channel} {self.timestamp} {self.data}"

@typechecked
class SensorTail(Iterator[Datagram]):
    """Class to watch live data from a sensor.

    Parameters
    ----------
    sensor_id : int
        The ID of the sensor to watch.
    api_url : str, optional
        The URL of the Shakemap API endpoint.
    """
    sensor_id: int
    api_url: str
    ws: WebSocket = WebSocket()

    def __init__(self, sensor_id: int, api_url: Optional[str] = None):
        self.sensor_id = sensor_id
        self.api_url = api_url or f"wss://crisislab-data.massey.ac.nz/consume/{self.sensor_id}/live"

    def __iter__(self) -> Iterator[Datagram]:
        self.ws.connect(self.api_url) # type: ignore
        return self

    def __next__(self) -> Datagram:
        data = self.ws.recv()
        parsed = msgpack.loads(data)

        if parsed["type"] == "datagram":
            # TODO: handle multiple datagrams in one message
            channel, timestamp, *data = parsed["data"][0]

            return Datagram(channel, timestamp, data)
        
        if parsed["type"] == "sensor-meta":
            self.sensor_meta = parsed["data"]
            if self.sensor_meta["online"] != True:
                raise Exception(f"Sensor #{self.sensor_id} seems to be offline.")
        
        return self.__next__()

    def __del__(self):
        self.ws.close() # type: ignore



from pysislab import get_sensors, SensorTail

def test_get_sensors():
    sensors = get_sensors()
    assert len(sensors) > 0
    assert sensors[0].id == 1
    assert sensors[0].secondary_id == "AM.R02C9"
    assert sensors[0].type == "Raspberry Shake 4D"

def test_sensor_tail():
    sensor_tail = SensorTail(22)

    it = sensor_tail.__iter__()

    assert it is sensor_tail

    datagram = sensor_tail.__next__()

    assert datagram.channel in ["EHZ", "ENN", "ENZ", "ENE"]
    assert datagram.timestamp > 0
    assert len(datagram.data) >= 25



from setuptools import setup, find_packages

# annotated-types==0.5.0
# certifi==2023.7.22
# charset-normalizer==3.2.0
# idna==3.4
# importlib-metadata==6.8.0
# markdown-it-py==3.0.0
# mdurl==0.1.2
# msgpack==1.0.5
# pydantic==2.3.0
# pydantic_core==2.6.3
# Pygments==2.16.1
# requests==2.31.0
# rich==13.5.2
# typeguard==4.1.5
# typing_extensions==4.7.1
# urllib3==2.0.4
# websocket-client==1.6.3
# zipp==3.16.2

setup(
   name='pysislab',
   description='Package to fetch data from the CRISiSLab seismometer network',
   author='Ben Hong',
   packages=find_packages(),
   entry_points={
      'console_scripts': [
         'pysislab=pysislab.cli:main',
      ],
   },
   install_requires=[
      'rich',
      'requests',
      'websocket-client',
      'msgpack',
      'pydantic',
      'typeguard',
   ],
)